import { ScopePropObject, createProvideScope } from '@oku-ui/provide';
import { defineComponent, toRefs, reactive, ref, computed, watchEffect, h, mergeProps, useModel } from 'vue';
import { composeEventHandlers } from '@oku-ui/utils';
import { reactiveOmit, usePrevious, useSize, useForwardRef, useComposedRefs, useControllable } from '@oku-ui/use-composable';
import { primitiveProps, Primitive } from '@oku-ui/primitive';
import { OkuPresence } from '@oku-ui/presence';

const scopeCheckboxProps = {
  scopeOkuCheckbox: {
    ...ScopePropObject
  }
};
function isIndeterminate(checked) {
  return checked === "indeterminate";
}
function getState(checked) {
  return isIndeterminate(checked) ? "indeterminate" : checked ? "checked" : "unchecked";
}

const bubbleInputProps = {
  props: {
    checked: {
      type: [Boolean, String, Number],
      required: true
    },
    control: {
      type: Object,
      default: null,
      required: true
    },
    bubbles: {
      type: Boolean,
      default: true,
      required: true
    }
  }
};
const bubbleInput = defineComponent({
  name: "OkuBubbleInput",
  inheritAttrs: false,
  props: {
    ...bubbleInputProps.props
  },
  setup(props, { attrs }) {
    const { checked, bubbles, control, ...inputProps } = toRefs(props);
    const _reactive = reactive(inputProps);
    const reactiveIputProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const _ref = ref();
    const prevChecked = usePrevious(checked);
    const controlSize = computed(() => useSize(control));
    watchEffect(() => {
      const input = _ref.value;
      const inputProto = window.HTMLInputElement.prototype;
      const descriptor = Object.getOwnPropertyDescriptor(inputProto, "checked");
      const setChecked = descriptor.set;
      if (prevChecked.value !== checked.value && setChecked) {
        const event = new Event("click", { bubbles: bubbles.value });
        input.indeterminate = isIndeterminate(checked.value);
        setChecked.call(input, isIndeterminate(checked.value) ? false : checked);
        input.dispatchEvent(event);
      }
    });
    return () => h("input", {
      "type": "checkbox",
      "aria-hidden": true,
      "defaultChecked": isIndeterminate(checked.value) ? false : checked.value,
      ...mergeProps(attrs, reactiveIputProps),
      "tabindex": -1,
      "ref": _ref,
      "style": {
        ...attrs.style,
        ...controlSize.value,
        position: "absolute",
        pointerEvents: "none",
        opacity: 0,
        margin: "0px"
      }
    });
  }
});
const OkuBubbleInput = bubbleInput;

const CHECKBOX_NAME = "OkuCheckbox";
const [createCheckboxProvider, createCheckboxScope] = createProvideScope(CHECKBOX_NAME);
const [CheckboxProvider, useCheckboxInject] = createCheckboxProvider(CHECKBOX_NAME);
const checkboxProps = {
  props: {
    modelValue: {
      type: [Boolean, String, Number, void 0],
      default: void 0
    },
    checked: {
      type: [Boolean, String, Number, void 0],
      default: void 0
    },
    defaultChecked: {
      type: [Boolean, String, void 0],
      default: void 0
    },
    required: {
      type: Boolean,
      default: void 0
    },
    name: {
      type: String,
      default: void 0
    },
    disabled: {
      type: Boolean,
      default: void 0
    },
    value: {
      type: String,
      default: void 0
    }
  },
  emits: {
    // eslint-disable-next-line unused-imports/no-unused-vars
    "update:modelValue": (checked) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "checkedChange": (checked) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "keydown": (event) => true,
    // eslint-disable-next-line unused-imports/no-unused-vars
    "click": (event) => true
  }
};
const checkboxDisplayName = "OkuCheckbox";
const Checkbox = defineComponent({
  name: checkboxDisplayName,
  components: { OkuBubbleInput },
  inheritAttrs: false,
  props: {
    ...checkboxProps.props,
    ...scopeCheckboxProps,
    ...primitiveProps
  },
  emits: checkboxProps.emits,
  setup(props, { attrs, slots, emit }) {
    const {
      modelValue: _modelValue,
      scopeOkuCheckbox,
      checked: checkedProp,
      defaultChecked,
      required,
      disabled,
      name,
      value,
      ...checkboxProps2
    } = toRefs(props);
    const _reactive = reactive(checkboxProps2);
    const reactiveCheckboxProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const buttonRef = ref(null);
    const forwardedRef = useForwardRef();
    const composedRefs = useComposedRefs(buttonRef, forwardedRef);
    const hasConsumerStoppedPropagationRef = ref(false);
    const modelValue = useModel(props, "modelValue");
    const proxyChecked = computed({
      get: () => modelValue.value !== void 0 ? modelValue.value : checkedProp.value !== void 0 ? checkedProp.value : void 0,
      set: () => {
      }
    });
    const { state, updateValue } = useControllable({
      prop: computed(() => proxyChecked.value),
      defaultProp: computed(() => defaultChecked.value),
      onChange: (result) => {
        modelValue.value = result;
        emit("checkedChange", result);
      },
      initialValue: false
    });
    const initialCheckedStateRef = ref(state.value);
    const isFormControl = computed(() => {
      return Boolean(buttonRef.value?.closest("form")) || false;
    });
    watchEffect(() => {
      const form = buttonRef.value?.form;
      if (form) {
        const reset = () => updateValue(initialCheckedStateRef.value);
        form.addEventListener("reset", reset);
        return () => form.removeEventListener("reset", reset);
      }
    });
    CheckboxProvider({
      scope: scopeOkuCheckbox.value,
      state,
      disabled
    });
    const originalReturn = () => [
      h(
        Primitive.button,
        {
          "type": "button",
          "role": "checkbox",
          "aria-checked": computed(() => isIndeterminate(state.value) ? "mixed" : state.value).value,
          "aria-required": required.value,
          "data-state": computed(() => getState(state.value)).value,
          "data-disabled": disabled.value ? "" : void 0,
          "disabled": disabled.value,
          "value": value.value,
          "asChild": props.asChild,
          ...mergeProps(attrs, reactiveCheckboxProps),
          "ref": composedRefs,
          "onKeydown": composeEventHandlers((e) => {
            emit("keydown", e);
          }, (event) => {
            if (event.key === "Enter")
              event.preventDefault();
          }),
          "onClick": composeEventHandlers((e) => {
            emit("click", e);
          }, (event) => {
            updateValue(isIndeterminate(state.value) ? true : !state.value);
            if (isFormControl.value) {
              if (!hasConsumerStoppedPropagationRef.value)
                event.stopPropagation();
            }
          })
        },
        {
          default: () => slots.default?.()
        }
      ),
      isFormControl.value && h(
        OkuBubbleInput,
        {
          bubbles: computed(() => !hasConsumerStoppedPropagationRef.value).value,
          name: name.value,
          value: value.value,
          checked: state.value,
          required: required.value,
          control: buttonRef.value,
          disabled: disabled.value,
          // We transform because the input is absolutely positioned but we have
          // rendered it **after** the button. This pulls it back to sit on top
          // of the button.
          style: { transform: "translateX(-100%)" }
        }
      )
    ];
    return originalReturn;
  }
});
const OkuCheckbox = Checkbox;

const checkboxIndicatorProps = {
  props: {
    forceMount: {
      type: Boolean,
      default: void 0
    }
  },
  emits: {}
};
const INDICATOR_NAME = "OkuCheckboxIndicator";
const checkboxIndicator = defineComponent({
  name: INDICATOR_NAME,
  props: {
    ...checkboxIndicatorProps.props,
    ...scopeCheckboxProps,
    ...primitiveProps
  },
  setup(props, { attrs, slots }) {
    const { forceMount, scopeOkuCheckbox, ...indicatorProps } = toRefs(props);
    const _reactive = reactive(indicatorProps);
    const reactiveIndicatorProps = reactiveOmit(_reactive, (key, _value) => key === void 0);
    const forwardedRef = useForwardRef();
    const context = useCheckboxInject(INDICATOR_NAME, scopeOkuCheckbox.value);
    const originalReturn = () => h(OkuPresence, {
      present: computed(() => forceMount.value || isIndeterminate(context.state.value) || context.state.value === true).value
    }, {
      default: () => h(
        Primitive.span,
        {
          "ref": forwardedRef,
          "data-state": computed(() => getState(context.state.value)).value,
          "data-disabled": context.disabled?.value ? "" : void 0,
          ...mergeProps(attrs, reactiveIndicatorProps),
          "style": { pointerEvents: "none", ...attrs.style }
        },
        {
          default: () => slots.default?.()
        }
      )
    });
    return originalReturn;
  }
});
const OkuCheckboxIndicator = checkboxIndicator;

export { OkuCheckbox, OkuCheckboxIndicator, checkboxIndicatorProps, checkboxProps, createCheckboxScope };
